@@ -27,6 +27,11 @@
  * - changing node-shape from box into record
  * - use labels like : element [ label="{element | <src> src | <sink> sink}"]
  * - point to record-connectors : element1:src -> element2:sink
+ * - we use head/tail labels for pad-caps right now
+ *   - this does not work well, as dot seems to not llok at their sizen when
+ *     doing the layout
+ *   - we could add the caps to the pad itself, then we should use one line per
+ *     caps (simple caps = one line)
  */
 
 #include "gst_private.h"
@@ -130,6 +135,55 @@ debug_dump_get_element_params (GstElement * element)
   return param_name;
 }
 
+static void
+debug_dump_pad (GstPad * pad, const gchar * color_name,
+    const gchar * element_name, GstDebugGraphDetails details, FILE * out,
+    const gint indent)
+{
+  GstPadTemplate *pad_templ;
+  GstPadPresence presence;
+  gchar *pad_name;
+  const gchar *style_name;
+  const gchar *spc = &spaces[MAX (sizeof (spaces) - (1 + indent * 2), 0)];
+
+  pad_name = debug_dump_make_object_name (GST_OBJECT (pad));
+
+  /* pad availability */
+  style_name = "filled,solid";
+  if ((pad_templ = gst_pad_get_pad_template (pad))) {
+    presence = GST_PAD_TEMPLATE_PRESENCE (pad_templ);
+    if (presence == GST_PAD_SOMETIMES) {
+      style_name = "filled,dotted";
+    } else if (presence == GST_PAD_REQUEST) {
+      style_name = "filled,dashed";
+    }
+  }
+  if (details & GST_DEBUG_GRAPH_SHOW_STATES) {
+    gchar pad_flags[6];
+    const gchar *activation_mode = "-><";
+
+    /* check if pad flags */
+    pad_flags[0] = GST_OBJECT_FLAG_IS_SET (pad, GST_PAD_BLOCKED) ? 'B' : 'b';
+    pad_flags[1] = GST_OBJECT_FLAG_IS_SET (pad, GST_PAD_FLUSHING) ? 'F' : 'f';
+    pad_flags[2] = GST_OBJECT_FLAG_IS_SET (pad, GST_PAD_IN_GETCAPS) ? 'G' : 'g';
+    pad_flags[3] = GST_OBJECT_FLAG_IS_SET (pad, GST_PAD_IN_SETCAPS) ? 'S' : 's';
+    pad_flags[4] = GST_OBJECT_FLAG_IS_SET (pad, GST_PAD_BLOCKING) ? 'B' : 'b';
+    pad_flags[5] = '\0';
+
+    fprintf (out,
+        "%s  %s_%s [color=black, fillcolor=\"%s\", label=\"%s\\n[%c][%s]\", height=\"0.2\", style=\"%s\"];\n",
+        spc, element_name, pad_name, color_name, GST_OBJECT_NAME (pad),
+        activation_mode[pad->mode], pad_flags, style_name);
+  } else {
+    fprintf (out,
+        "%s  %s_%s [color=black, fillcolor=\"%s\", label=\"%s\", height=\"0.2\", style=\"%s\"];\n",
+        spc, element_name, pad_name, color_name, GST_OBJECT_NAME (pad),
+        style_name);
+  }
+
+  g_free (pad_name);
+}
+
 static void
 debug_dump_element_pad (GstPad * pad, GstElement * element,
     GstDebugGraphDetails details, FILE * out, const gint indent)
@@ -137,15 +191,11 @@ debug_dump_element_pad (GstPad * pad, GstElement * element,
   GstElement *target_element;
   GstPad *target_pad, *tmp_pad;
   GstPadDirection dir;
-  GstPadTemplate *pad_templ;
-  GstPadPresence presence;
-  gchar *pad_name, *element_name;
-  gchar *target_pad_name, *target_element_name;
-  gchar *color_name, *style_name;
-  const gchar *spc = &spaces[MAX (sizeof (spaces) - (1 + indent * 2), 0)];
+  gchar *element_name;
+  gchar *target_element_name;
+  const gchar *color_name;
 
   dir = gst_pad_get_direction (pad);
-  pad_name = debug_dump_make_object_name (GST_OBJECT (pad));
   element_name = debug_dump_make_object_name (GST_OBJECT (element));
   if (GST_IS_GHOST_PAD (pad)) {
     color_name =
@@ -154,31 +204,17 @@ debug_dump_element_pad (GstPad * pad, GstElement * element,
     /* output target-pad so that it belongs to this element */
     if ((tmp_pad = gst_ghost_pad_get_target (GST_GHOST_PAD (pad)))) {
       if ((target_pad = gst_pad_get_peer (tmp_pad))) {
-        target_pad_name = debug_dump_make_object_name (GST_OBJECT (target_pad));
         if ((target_element = gst_pad_get_parent_element (target_pad))) {
           target_element_name =
               debug_dump_make_object_name (GST_OBJECT (target_element));
         } else {
-          target_element_name = "";
+          target_element_name = g_strdup ("");
         }
-        style_name = "filled,solid";
-        if ((pad_templ = gst_pad_get_pad_template (target_pad))) {
-          presence = GST_PAD_TEMPLATE_PRESENCE (pad_templ);
-          if (presence == GST_PAD_SOMETIMES) {
-            style_name = "filled,dotted";
-          } else if (presence == GST_PAD_REQUEST) {
-            style_name = "filled,dashed";
-          }
-        }
-        fprintf (out,
-            "%s  %s_%s [color=black, fillcolor=\"%s\", label=\"%s\", height=\"0.2\", style=\"%s\"];\n",
-            spc, target_element_name, target_pad_name, color_name,
-            GST_OBJECT_NAME (target_pad), style_name);
-        g_free (target_pad_name);
-        if (target_element) {
-          g_free (target_element_name);
+        debug_dump_pad (target_pad, color_name, target_element_name, details,
+            out, indent);
+        g_free (target_element_name);
+        if (target_element)
           gst_object_unref (target_element);
-        }
         gst_object_unref (target_pad);
       }
       gst_object_unref (tmp_pad);
@@ -189,21 +225,7 @@ debug_dump_element_pad (GstPad * pad, GstElement * element,
             GST_PAD_SINK) ? "#aaaaff" : "#cccccc");
   }
   /* pads */
-  style_name = "filled,solid";
-  if ((pad_templ = gst_pad_get_pad_template (pad))) {
-    presence = GST_PAD_TEMPLATE_PRESENCE (pad_templ);
-    if (presence == GST_PAD_SOMETIMES) {
-      style_name = "filled,dotted";
-    } else if (presence == GST_PAD_REQUEST) {
-      style_name = "filled,dashed";
-    }
-  }
-  fprintf (out,
-      "%s  %s_%s [color=black, fillcolor=\"%s\", label=\"%s\", height=\"0.2\", style=\"%s\"];\n",
-      spc, element_name, pad_name, color_name, GST_OBJECT_NAME (pad),
-      style_name);
-
-  g_free (pad_name);
+  debug_dump_pad (pad, color_name, element_name, details, out, indent);
   g_free (element_name);
 }
 
@@ -244,8 +266,7 @@ string_append_field (GQuark field, const GValue * value, gpointer ptr)
 }
 
 static gchar *
-debug_dump_describe_caps (GstCaps * caps, GstDebugGraphDetails details,
-    gboolean * need_free)
+debug_dump_describe_caps (GstCaps * caps, GstDebugGraphDetails details)
 {
   gchar *media = NULL;
 
@@ -253,7 +274,6 @@ debug_dump_describe_caps (GstCaps * caps, GstDebugGraphDetails details,
 
     if (gst_caps_is_any (caps) || gst_caps_is_empty (caps)) {
       media = gst_caps_to_string (caps);
-      *need_free = TRUE;
 
     } else {
       GString *str = NULL;
@@ -276,16 +296,14 @@ debug_dump_describe_caps (GstCaps * caps, GstDebugGraphDetails details,
       }
 
       media = g_string_free (str, FALSE);
-      *need_free = TRUE;
     }
 
   } else {
     if (GST_CAPS_IS_SIMPLE (caps))
       media =
-          (gchar *) gst_structure_get_name (gst_caps_get_structure (caps, 0));
+          g_strdup (gst_structure_get_name (gst_caps_get_structure (caps, 0)));
     else
-      media = "*";
-    *need_free = FALSE;
+      media = g_strdup ("*");
   }
   return media;
 }
@@ -297,8 +315,6 @@ debug_dump_element_pad_link (GstPad * pad, GstElement * element,
   GstElement *peer_element, *target_element;
   GstPad *peer_pad, *target_pad, *tmp_pad;
   GstCaps *caps, *peer_caps;
-  gboolean free_caps, free_peer_caps;
-  gboolean free_media, free_media_src, free_media_sink;
   gchar *media = NULL;
   gchar *media_src = NULL, *media_sink = NULL;
   gchar *pad_name, *element_name;
@@ -307,69 +323,47 @@ debug_dump_element_pad_link (GstPad * pad, GstElement * element,
   const gchar *spc = &spaces[MAX (sizeof (spaces) - (1 + indent * 2), 0)];
 
   if ((peer_pad = gst_pad_get_peer (pad))) {
-    free_media = free_media_src = free_media_sink = FALSE;
     if ((details & GST_DEBUG_GRAPH_SHOW_MEDIA_TYPE) ||
         (details & GST_DEBUG_GRAPH_SHOW_CAPS_DETAILS)
         ) {
-      if ((caps = gst_pad_get_negotiated_caps (pad))) {
-        free_caps = TRUE;
-      } else {
-        free_caps = FALSE;
-        if (!(caps = (GstCaps *)
-                gst_pad_get_pad_template_caps (pad))) {
-          /* this should not happen */
-          media = "?";
-        }
-      }
-      if ((peer_caps = gst_pad_get_negotiated_caps (peer_pad))) {
-        free_peer_caps = TRUE;
-      } else {
-        free_peer_caps = FALSE;
-        peer_caps = (GstCaps *) gst_pad_get_pad_template_caps (peer_pad);
-      }
-      if (caps) {
-        media = debug_dump_describe_caps (caps, details, &free_media);
-        /* check if peer caps are different */
-        if (peer_caps && !gst_caps_is_equal (caps, peer_caps)) {
-          gchar *tmp;
-          gboolean free_tmp;
+      caps = gst_pad_get_negotiated_caps (pad);
+      if (!caps)
+        caps = gst_caps_copy (gst_pad_get_pad_template_caps (pad));
+      peer_caps = gst_pad_get_negotiated_caps (peer_pad);
+      if (!peer_caps)
+        peer_caps = gst_caps_copy (gst_pad_get_pad_template_caps (peer_pad));
 
-          tmp = debug_dump_describe_caps (peer_caps, details, &free_tmp);
-          if (gst_pad_get_direction (pad) == GST_PAD_SRC) {
-            media_src = media;
-            free_media_src = free_media;
-            media_sink = tmp;
-            free_media_sink = free_tmp;
-          } else {
-            media_src = tmp;
-            free_media_src = free_tmp;
-            media_sink = media;
-            free_media_sink = free_media;
-          }
-          media = NULL;
-          free_media = FALSE;
-        }
-        if (free_caps) {
-          gst_caps_unref (caps);
+      media = debug_dump_describe_caps (caps, details);
+      /* check if peer caps are different */
+      if (peer_caps && !gst_caps_is_equal (caps, peer_caps)) {
+        gchar *tmp;
+
+        tmp = debug_dump_describe_caps (peer_caps, details);
+        if (gst_pad_get_direction (pad) == GST_PAD_SRC) {
+          media_src = media;
+          media_sink = tmp;
+        } else {
+          media_src = tmp;
+          media_sink = media;
         }
+        media = NULL;
       }
-      if (free_peer_caps && peer_caps) {
-        gst_caps_unref (peer_caps);
-      }
+      gst_caps_unref (peer_caps);
+      gst_caps_unref (caps);
     }
 
     pad_name = debug_dump_make_object_name (GST_OBJECT (pad));
     if (element) {
       element_name = debug_dump_make_object_name (GST_OBJECT (element));
     } else {
-      element_name = "";
+      element_name = g_strdup ("");
     }
     peer_pad_name = debug_dump_make_object_name (GST_OBJECT (peer_pad));
     if ((peer_element = gst_pad_get_parent_element (peer_pad))) {
       peer_element_name =
           debug_dump_make_object_name (GST_OBJECT (peer_element));
     } else {
-      peer_element_name = "";
+      peer_element_name = g_strdup ("");
     }
 
     if (GST_IS_GHOST_PAD (pad)) {
@@ -381,17 +375,16 @@ debug_dump_element_pad_link (GstPad * pad, GstElement * element,
             target_element_name =
                 debug_dump_make_object_name (GST_OBJECT (target_element));
           } else {
-            target_element_name = "";
+            target_element_name = g_strdup ("");
           }
           /* src ghostpad relationship */
           fprintf (out, "%s%s_%s -> %s_%s [style=dashed, minlen=0]\n", spc,
               target_element_name, target_pad_name, element_name, pad_name);
 
           g_free (target_pad_name);
-          if (target_element) {
-            g_free (target_element_name);
+          g_free (target_element_name);
+          if (target_element)
             gst_object_unref (target_element);
-          }
           gst_object_unref (target_pad);
         }
         gst_object_unref (tmp_pad);
@@ -406,7 +399,7 @@ debug_dump_element_pad_link (GstPad * pad, GstElement * element,
             target_element_name =
                 debug_dump_make_object_name (GST_OBJECT (target_element));
           } else {
-            target_element_name = "";
+            target_element_name = g_strdup ("");
           }
           /* sink ghostpad relationship */
           fprintf (out, "%s%s_%s -> %s_%s [style=dashed, minlen=0]\n", spc,
@@ -421,10 +414,9 @@ debug_dump_element_pad_link (GstPad * pad, GstElement * element,
           debug_dump_element_pad_link (target_pad, target_element, details, out,
               indent);
           g_free (target_pad_name);
-          if (target_element) {
-            g_free (target_element_name);
+          g_free (target_element_name);
+          if (target_element)
             gst_object_unref (target_element);
-          }
           gst_object_unref (target_pad);
         }
         gst_object_unref (tmp_pad);
@@ -435,35 +427,28 @@ debug_dump_element_pad_link (GstPad * pad, GstElement * element,
     if (media) {
       fprintf (out, "%s%s_%s -> %s_%s [label=\"%s\"]\n", spc,
           element_name, pad_name, peer_element_name, peer_pad_name, media);
-      if (free_media) {
-        g_free (media);
-      }
+      g_free (media);
     } else if (media_src && media_sink) {
       /* dot has some issues with placement of head and taillabels,
        * we need an empty label to make space */
-      fprintf (out, "%s%s_%s -> %s_%s [labeldistance=\"5\", labelangle=\"0\", "
-          "label=\"                         \", "
+      fprintf (out, "%s%s_%s -> %s_%s [labeldistance=\"10\", labelangle=\"0\", "
+          "label=\"                                                  \", "
           "headlabel=\"%s\", taillabel=\"%s\"]\n",
           spc, element_name, pad_name, peer_element_name, peer_pad_name,
           media_src, media_sink);
-      if (free_media_src)
-        g_free (media_src);
-      if (free_media_sink)
-        g_free (media_sink);
+      g_free (media_src);
+      g_free (media_sink);
     } else {
       fprintf (out, "%s%s_%s -> %s_%s\n", spc,
           element_name, pad_name, peer_element_name, peer_pad_name);
     }
 
     g_free (pad_name);
-    if (element) {
-      g_free (element_name);
-    }
+    g_free (element_name);
     g_free (peer_pad_name);
-    if (peer_element) {
-      g_free (peer_element_name);
+    g_free (peer_element_name);
+    if (peer_element)
       gst_object_unref (peer_element);
-    }
     gst_object_unref (peer_pad);
   }
 }
@@ -650,13 +635,13 @@ _gst_debug_bin_to_dot_file (GstBin * bin, GstDebugGraphDetails details,
         "digraph pipeline {\n"
         "  rankdir=LR;\n"
         "  fontname=\"sans\";\n"
-        "  fontsize=\"8\";\n"
+        "  fontsize=\"10\";\n"
         "  labelloc=t;\n"
         "  nodesep=.1;\n"
         "  ranksep=.2;\n"
         "  label=\"<%s>\\n%s%s%s\";\n"
-        "  node [style=filled, shape=box, fontsize=\"7\", fontname=\"sans\", margin=\"0.0,0.0\"];\n"
-        "  edge [labelfontsize=\"7\", fontsize=\"7\", fontname=\"monospace\"];\n"
+        "  node [style=filled, shape=box, fontsize=\"9\", fontname=\"sans\", margin=\"0.0,0.0\"];\n"
+        "  edge [labelfontsize=\"6\", fontsize=\"9\", fontname=\"monospace\"];\n"
         "\n", G_OBJECT_TYPE_NAME (bin), GST_OBJECT_NAME (bin),
         (state_name ? state_name : ""), (param_name ? param_name : "")
         );
