@@ -35,8 +35,8 @@ void		gst_util_set_value_from_string	(GValue *value, const gchar *value_str);
 void		gst_util_set_object_arg		(GObject *object, const gchar *name, const gchar *value);
 void		gst_util_dump_mem		(const guchar *mem, guint size);
 
-guint64         gst_util_gdouble_to_guint64     (gdouble value)  G_GNUC_PURE;
-gdouble         gst_util_guint64_to_gdouble     (guint64 value)  G_GNUC_PURE;
+guint64         gst_util_gdouble_to_guint64     (gdouble value)  G_GNUC_CONST;
+gdouble         gst_util_guint64_to_gdouble     (guint64 value)  G_GNUC_CONST;
 
 /**
  * gst_guint64_to_gdouble:
@@ -63,9 +63,13 @@ gdouble         gst_util_guint64_to_gdouble     (guint64 value)  G_GNUC_PURE;
 #define         gst_guint64_to_gdouble(value)   ((gdouble) (value))
 #endif
 
-guint64		gst_util_uint64_scale		(guint64 val, guint64 num, guint64 denom) G_GNUC_PURE;
+guint64		gst_util_uint64_scale		(guint64 val, guint64 num, guint64 denom) G_GNUC_CONST;
+guint64		gst_util_uint64_scale_round	(guint64 val, guint64 num, guint64 denom) G_GNUC_CONST;
+guint64		gst_util_uint64_scale_ceil	(guint64 val, guint64 num, guint64 denom) G_GNUC_CONST;
 
-guint64         gst_util_uint64_scale_int       (guint64 val, gint num, gint denom) G_GNUC_PURE;
+guint64         gst_util_uint64_scale_int       (guint64 val, gint num, gint denom) G_GNUC_CONST;
+guint64         gst_util_uint64_scale_int_round (guint64 val, gint num, gint denom) G_GNUC_CONST;
+guint64         gst_util_uint64_scale_int_ceil  (guint64 val, gint num, gint denom) G_GNUC_CONST;
 
 guint32         gst_util_seqnum_next            (void);
 gint32          gst_util_seqnum_compare         (guint32 s1, guint32 s2);
@@ -127,8 +131,6 @@ type_as_function ## _class_init_trampoline (gpointer g_class,		\
   type_as_function ## _class_init ((type ## Class *)g_class);		\
 }									\
 									\
-GType type_as_function ## _get_type (void);				\
-									\
 GType									\
 type_as_function ## _get_type (void)					\
 {									\
@@ -268,10 +270,10 @@ GST_BOILERPLATE_FULL (type, type_as_function, parent_type,              \
 
 /* Define PUT and GET functions for unaligned memory */
 #define _GST_GET(__data, __idx, __size, __shift) \
-    (((guint##__size) (((guint8 *) (__data))[__idx])) << __shift)
+    (((guint##__size) (((guint8 *) (__data))[__idx])) << (__shift))
 
 #define _GST_PUT(__data, __idx, __size, __shift, __num) \
-    (((guint8 *) (__data))[__idx] = (((guint##__size) __num) >> __shift) & 0xff)
+    (((guint8 *) (__data))[__idx] = (((guint##__size) (__num)) >> (__shift)) & 0xff)
 
 /**
  * GST_READ_UINT64_BE:
@@ -514,6 +516,8 @@ GST_BOILERPLATE_FULL (type, type_as_function, parent_type,              \
  *
  * Swap byte order of a 32-bit floating point value (float).
  *
+ * Returns: @in byte-swapped.
+ *
  * Since: 0.10.22
  *
  */
@@ -541,6 +545,8 @@ GFLOAT_SWAP_LE_BE(gfloat in)
  *
  * Swap byte order of a 64-bit floating point value (double).
  *
+ * Returns: @in byte-swapped.
+ *
  * Since: 0.10.22
  *
  */
@@ -673,6 +679,8 @@ GDOUBLE_SWAP_LE_BE(gdouble in)
  *
  * Read a 32 bit float value in little endian format from the memory buffer.
  *
+ * Returns: The floating point value read from @data
+ *
  * Since: 0.10.22
  *
  */
@@ -699,6 +707,8 @@ GST_READ_FLOAT_LE(const guint8 *data)
  *
  * Read a 32 bit float value in big endian format from the memory buffer.
  *
+ * Returns: The floating point value read from @data
+ *
  * Since: 0.10.22
  *
  */
@@ -725,6 +735,8 @@ GST_READ_FLOAT_BE(const guint8 *data)
  *
  * Read a 64 bit double value in little endian format from the memory buffer.
  *
+ * Returns: The double-precision floating point value read from @data
+ *
  * Since: 0.10.22
  *
  */
@@ -751,6 +763,8 @@ GST_READ_DOUBLE_LE(const guint8 *data)
  *
  * Read a 64 bit double value in big endian format from the memory buffer.
  *
+ * Returns: The double-precision floating point value read from @data
+ *
  * Since: 0.10.22
  *
  */
@@ -1150,6 +1164,25 @@ gpointer                gst_util_array_binary_search      (gpointer array, guint
 							   GstSearchMode mode, gconstpointer search_data,
 							   gpointer user_data);
 
+gint gst_util_greatest_common_divisor (gint a, gint b);
+void gst_util_fraction_to_double (gint src_n, gint src_d, gdouble *dest);
+void gst_util_double_to_fraction (gdouble src, gint *dest_n, gint *dest_d);
+gboolean gst_util_fraction_multiply (gint a_n, gint a_d, gint b_n, gint b_d, gint *res_n, gint *res_d);
+gboolean gst_util_fraction_add (gint a_n, gint a_d, gint b_n, gint b_d, gint *res_n, gint *res_d);
+
+
+/* sink message event
+ *
+ * FIXME: This should be in gstevent.h but can't because
+ * it needs GstMessage and this would introduce circular
+ * header includes. And forward declarations of typedefs
+ * are unfortunately not possible. The implementation of
+ * these functions is in gstevent.c.
+ */
+GstEvent*       gst_event_new_sink_message      (GstMessage *msg);
+void            gst_event_parse_sink_message    (GstEvent *event, GstMessage **msg);
+
+
 G_END_DECLS
 
 #endif /* __GST_UTILS_H__ */
