@@ -180,15 +180,12 @@ gst_test_mono_source_class_init (GstTestMonoSourceClass * klass)
 static void
 gst_test_mono_source_base_init (GstTestMonoSourceClass * klass)
 {
-  static const GstElementDetails details = {
-    "Monophonic source for unit tests",
-    "Source/Audio/MonoSource",
-    "Use in unit tests",
-    "Stefan Kost <ensonic@users.sf.net>"
-  };
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
-  gst_element_class_set_details (element_class, &details);
+  gst_element_class_set_details_simple (element_class,
+      "Monophonic source for unit tests",
+      "Source/Audio/MonoSource",
+      "Use in unit tests", "Stefan Kost <ensonic@users.sf.net>");
 }
 
 static GType
@@ -535,6 +532,78 @@ GST_START_TEST (controller_controlsource_refcounts)
 
 GST_END_TEST;
 
+/* tests if we don't fail on empty controllers */
+GST_START_TEST (controller_controlsource_empty1)
+{
+  GstController *ctrl;
+  GstElement *elem;
+  GstControlSource *csource;
+
+  gst_controller_init (NULL, NULL);
+
+  elem = gst_element_factory_make ("testmonosource", "test_source");
+
+  /* that property should exist and should be controllable */
+  ctrl = gst_controller_new (G_OBJECT (elem), "ulong", NULL);
+  fail_unless (ctrl != NULL, NULL);
+
+  csource = (GstControlSource *) gst_interpolation_control_source_new ();
+  fail_unless (csource != NULL, NULL);
+
+  fail_unless (gst_controller_set_control_source (ctrl, "ulong", csource));
+
+  /* don't fail on empty control point lists */
+  gst_controller_sync_values (ctrl, 0 * GST_SECOND);
+
+  /* unref objects */
+  g_object_unref (csource);
+  g_object_unref (ctrl);
+  gst_object_unref (elem);
+}
+
+GST_END_TEST;
+
+/* tests if we don't fail on controllers that are empty again */
+GST_START_TEST (controller_controlsource_empty2)
+{
+  GstController *ctrl;
+  GstElement *elem;
+  GstInterpolationControlSource *csource;
+  GValue val = { 0, };
+
+  gst_controller_init (NULL, NULL);
+
+  elem = gst_element_factory_make ("testmonosource", "test_source");
+
+  /* that property should exist and should be controllable */
+  ctrl = gst_controller_new (G_OBJECT (elem), "ulong", NULL);
+  fail_unless (ctrl != NULL, NULL);
+
+  csource = gst_interpolation_control_source_new ();
+  fail_unless (csource != NULL, NULL);
+
+  fail_unless (gst_controller_set_control_source (ctrl, "ulong",
+          (GstControlSource *) csource));
+
+  /* set control values */
+  g_value_init (&val, G_TYPE_ULONG);
+  g_value_set_ulong (&val, 0);
+  gst_interpolation_control_source_set (csource, 0 * GST_SECOND, &val);
+
+  /* ... and unset the value */
+  gst_interpolation_control_source_unset (csource, 0 * GST_SECOND);
+
+  /* don't fail on empty control point lists */
+  gst_controller_sync_values (ctrl, 0 * GST_SECOND);
+
+  /* unref objects */
+  g_object_unref (csource);
+  g_object_unref (ctrl);
+  gst_object_unref (elem);
+}
+
+GST_END_TEST;
+
 /* test timed value handling without interpolation */
 GST_START_TEST (controller_interpolate_none)
 {
@@ -797,6 +866,66 @@ GST_START_TEST (controller_interpolate_cubic)
 
 GST_END_TEST;
 
+/* test timed value handling with cubic interpolation */
+GST_START_TEST (controller_interpolate_cubic_too_few_cp)
+{
+  GstController *ctrl;
+  GstInterpolationControlSource *csource;
+  GstElement *elem;
+  gboolean res;
+  GValue val_double = { 0, };
+
+  gst_controller_init (NULL, NULL);
+
+  elem = gst_element_factory_make ("testmonosource", "test_source");
+
+  /* that property should exist and should be controllable */
+  ctrl = gst_controller_new (G_OBJECT (elem), "double", NULL);
+  fail_unless (ctrl != NULL, NULL);
+
+  /* Get interpolation control source */
+  csource = gst_interpolation_control_source_new ();
+
+  fail_unless (csource != NULL);
+  fail_unless (gst_controller_set_control_source (ctrl, "double",
+          GST_CONTROL_SOURCE (csource)));
+
+  /* set interpolation mode */
+  fail_unless (gst_interpolation_control_source_set_interpolation_mode (csource,
+          GST_INTERPOLATE_CUBIC));
+
+  /* set 2 control values */
+  g_value_init (&val_double, G_TYPE_DOUBLE);
+  g_value_set_double (&val_double, 0.0);
+  res =
+      gst_interpolation_control_source_set (csource, 0 * GST_SECOND,
+      &val_double);
+  fail_unless (res, NULL);
+  g_value_set_double (&val_double, 4.0);
+  res =
+      gst_interpolation_control_source_set (csource, 2 * GST_SECOND,
+      &val_double);
+  fail_unless (res, NULL);
+
+  g_object_unref (G_OBJECT (csource));
+
+  /* now pull in values for some timestamps and verify that it used linear
+   * interpolation as we don't gave enough control points
+   */
+  gst_controller_sync_values (ctrl, 0 * GST_SECOND);
+  fail_unless_equals_float (GST_TEST_MONO_SOURCE (elem)->val_double, 0.0);
+  gst_controller_sync_values (ctrl, 1 * GST_SECOND);
+  fail_unless_equals_float (GST_TEST_MONO_SOURCE (elem)->val_double, 2.0);
+  gst_controller_sync_values (ctrl, 2 * GST_SECOND);
+  fail_unless_equals_float (GST_TEST_MONO_SOURCE (elem)->val_double, 4.0);
+
+  GST_INFO ("controller->ref_count=%d", G_OBJECT (ctrl)->ref_count);
+  g_object_unref (ctrl);
+  gst_object_unref (elem);
+}
+
+GST_END_TEST;
+
 /* make sure we don't crash when someone sets an unsupported interpolation
  * mode */
 GST_START_TEST (controller_interpolate_unimplemented)
@@ -890,14 +1019,22 @@ GST_START_TEST (controller_interpolation_unset)
   res = gst_interpolation_control_source_unset (csource, 1 * GST_SECOND);
   fail_unless (res, NULL);
 
-  g_object_unref (csource);
-
   /* verify value again */
   gst_controller_sync_values (ctrl, 1 * GST_SECOND);
   fail_unless_equals_int (GST_TEST_MONO_SOURCE (elem)->val_ulong, 0);
   gst_controller_sync_values (ctrl, 2 * GST_SECOND);
   fail_unless_equals_int (GST_TEST_MONO_SOURCE (elem)->val_ulong, 50);
 
+  /* unset all values, reset and try to unset again */
+  fail_unless (gst_interpolation_control_source_unset (csource,
+          0 * GST_SECOND));
+  fail_unless (gst_interpolation_control_source_unset (csource,
+          2 * GST_SECOND));
+  gst_interpolation_control_source_unset_all (csource);
+  fail_if (gst_interpolation_control_source_unset (csource, 2 * GST_SECOND));
+
+  g_object_unref (csource);
+
   GST_INFO ("controller->ref_count=%d", G_OBJECT (ctrl)->ref_count);
   g_object_unref (ctrl);
   gst_object_unref (elem);
@@ -1011,7 +1148,7 @@ GST_START_TEST (controller_interpolation_linear_value_array)
   fail_unless (res, NULL);
 
   /* now pull in values for some timestamps */
-  values.property_name = "ulong";
+  values.property_name = (char *) "ulong";
   values.nbsamples = 3;
   values.sample_interval = GST_SECOND;
   values.values = (gpointer) g_new (gulong, 3);
@@ -1919,7 +2056,7 @@ GST_START_TEST (controller_refcount_new_list)
 
   /* that property should exist and should be controllable */
   elem = gst_element_factory_make ("testmonosource", "test_source");
-  list = g_list_append (NULL, "ulong");
+  list = g_list_append (NULL, (char *) "ulong");
   ctrl = gst_controller_new_list (G_OBJECT (elem), list);
   fail_unless (ctrl != NULL, NULL);
   GST_INFO ("controller->ref_count=%d", G_OBJECT (ctrl)->ref_count);
@@ -1930,8 +2067,8 @@ GST_START_TEST (controller_refcount_new_list)
 
   /* try the same property twice, make sure the refcount is still 1 */
   elem = gst_element_factory_make ("testmonosource", "test_source");
-  list = g_list_append (NULL, "ulong");
-  list = g_list_append (list, "ulong");
+  list = g_list_append (NULL, (char *) "ulong");
+  list = g_list_append (list, (char *) "ulong");
   ctrl = gst_controller_new_list (G_OBJECT (elem), list);
   fail_unless (ctrl != NULL, NULL);
   GST_INFO ("controller->ref_count=%d", G_OBJECT (ctrl)->ref_count);
@@ -1942,8 +2079,8 @@ GST_START_TEST (controller_refcount_new_list)
 
   /* try two properties, make sure the refcount is still 1 */
   elem = gst_element_factory_make ("testmonosource", "test_source");
-  list = g_list_append (NULL, "ulong");
-  list = g_list_append (list, "boolean");
+  list = g_list_append (NULL, (char *) "ulong");
+  list = g_list_append (list, (char *) "boolean");
   ctrl = gst_controller_new_list (G_OBJECT (elem), list);
   fail_unless (ctrl != NULL, NULL);
   GST_INFO ("controller->ref_count=%d", G_OBJECT (ctrl)->ref_count);
@@ -1957,7 +2094,7 @@ GST_START_TEST (controller_refcount_new_list)
   ctrl = gst_controller_new (G_OBJECT (elem), "ulong", NULL);
   fail_unless (ctrl != NULL, NULL);
   GST_INFO ("controller->ref_count=%d", G_OBJECT (ctrl)->ref_count);
-  list = g_list_append (NULL, "ulong");
+  list = g_list_append (NULL, (char *) "ulong");
   ctrl2 = gst_controller_new_list (G_OBJECT (elem), list);
   fail_unless (ctrl2 != NULL, NULL);
   fail_unless (ctrl == ctrl2, NULL);
@@ -1970,7 +2107,7 @@ GST_START_TEST (controller_refcount_new_list)
 
   /* try _new_list first and then _new */
   elem = gst_element_factory_make ("testmonosource", "test_source");
-  list = g_list_append (NULL, "ulong");
+  list = g_list_append (NULL, (char *) "ulong");
   ctrl = gst_controller_new_list (G_OBJECT (elem), list);
   fail_unless (ctrl != NULL, NULL);
   GST_INFO ("controller->ref_count=%d", G_OBJECT (ctrl)->ref_count);
@@ -2135,10 +2272,13 @@ gst_controller_suite (void)
   tcase_add_test (tc, controller_param_twice);
   tcase_add_test (tc, controller_finalize);
   tcase_add_test (tc, controller_controlsource_refcounts);
+  tcase_add_test (tc, controller_controlsource_empty1);
+  tcase_add_test (tc, controller_controlsource_empty2);
   tcase_add_test (tc, controller_interpolate_none);
   tcase_add_test (tc, controller_interpolate_trigger);
   tcase_add_test (tc, controller_interpolate_linear);
   tcase_add_test (tc, controller_interpolate_cubic);
+  tcase_add_test (tc, controller_interpolate_cubic_too_few_cp);
   tcase_add_test (tc, controller_interpolate_unimplemented);
   tcase_add_test (tc, controller_interpolation_unset);
   tcase_add_test (tc, controller_interpolation_unset_all);
